@@ -57,11 +57,13 @@ module Lens.Micro
   (^..), toListOf,
   (^?),
   (^?!),
+  traverseOf_,
   has,
   folded,
   folding,
 
   -- * Lens: a combined getter-and-setter
+  -- $lenses-note
   Lens, Lens',
   lens,
   at,
@@ -73,7 +75,9 @@ module Lens.Micro
   non,
 
   -- * Traversal: a lens iterating over several elements
+  -- $traversals-note
   Traversal, Traversal',
+  traverseOf,
   singular,
   failing,
   filtered,
@@ -587,6 +591,21 @@ s ^?! l = foldrOf l const (error "(^?!): empty Fold") s
 
 infixl 8 ^?!
 
+{- |
+Apply an action to all targets and discard the result (like 'Control.Monad.mapM_' or 'Data.Foldable.traverse_'):
+
+>>> traverseOf_ both putStrLn ("hello", "world")
+hello
+world
+
+Works with anything that allows getting, including lenses and getters (so, anything except for 'ASetter'). Should be faster than 'traverseOf' when you don't need the result.
+-}
+traverseOf_
+  :: Functor f
+  => Getting (Traversed r f) s a -> (a -> f r) -> s -> f ()
+traverseOf_ l f = void . getTraversed #. foldMapOf l (Traversed #. f)
+{-# INLINE traverseOf_ #-}
+
 {- |
 'has' checks whether a getter (any getter, including lenses, traversals, and folds) returns at least 1 value.
 
@@ -616,6 +635,97 @@ folding sfa agb = phantom . F.traverse_ agb . sfa
 
 -- Lenses ------------------------------------------------------------------
 
+{- $lenses-note
+
+Lenses are composable “pointers” at values inside some bigger structure (e.g. '_1' points at the first element of a tuple). You can use ('^.') to get, ('.~') to set, and ('%~') to modify:
+
+>>> (1,2) ^. _1
+1
+>>> (1,2) & _1 .~ 3
+(3,2)
+>>> (1,2) & _1 %~ negate
+(-1,2)
+
+To apply a monadic action (or an 'Applicative' action, or even a 'Functor' action) to the pointed value, just apply the lens directly or use 'traverseOf' (or 'traverseOf_' if you don't need the result):
+
+>>> traverseOf_ _1 print (1,2)
+1
+
+>>> _1 id (Just 1, 2)
+Just (1, 2)
+>>> _1 id (Nothing, 2)
+Nothing
+
+A 'Lens' can only point at a single value inside a structure (unlike a 'Traversal').
+
+('.') composes lenses (i.e. if a @B@ is a part of @A@, and a @C@ is a part of @B@, then @b.c@ lets you operate on @C@ inside @A@). You can create lenses with 'lens', or you can write them by hand.
+
+There are several ways to get lenses for some datatype:
+
+* They can already be provided by the package, by @microlens@, or by some other package like <http://hackage.haskell.org/package/microlens-platform microlens-platform>.
+
+* They can be provided by some unofficial package (like <http://hackage.haskell.org/package/microlens-aeson microlens-aeson>).
+
+* You can get them by combining already existing lenses.
+
+* You can derive them with Template Haskell (with <http://hackage.haskell.org/package/microlens-th microlens-th>).
+
+* You can write them with 'lens' if you have a setter and a getter. It's a simple and good way.
+
+* You can write them manually (sometimes it looks a bit better than the variant with 'lens', sometimes worse). The generic template is as follows:
+
+@
+somelens :: Lens s t a b
+
+-- “f” is the “a -> f b” function, “s” is the structure.
+somelens f s =
+  let
+    a = ...                 -- Extract the value from “s”.
+    rebuildWith b = ...     -- Write a function which would
+                            -- combine “s” and modified value
+                            -- to produce new structure.
+  in
+    rebuildWith '<$>' f a     -- Apply the structure-producing
+                            -- function to the modified value.
+@
+
+Here's the '_1' lens, for instance:
+
+@
+'_1' :: 'Lens' (a, x) (b, x) a b
+'_1' f (a, x) = (\\b -> (b, x)) '<$>' f a
+@
+
+Here's a more complicated lens, which extracts /several/ values from a structure (in a tuple):
+
+@
+type Age     = Int
+type City    = String
+type Country = String
+
+data Person = Person Age City Country
+
+-- This lens lets you access all location-related information about a person.
+location :: 'Lens'' Person (City, Country)
+location f (Person age city country) =
+  (\\(city', country') -> Person age city' country') '<$>' f (city, country)
+@
+
+You even can choose to use a lens to present /all/ information contained in the structure (in a different way). Such lenses are called @<http://hackage.haskell.org/package/lens/docs/Control-Lens-Iso.html#t:Iso Iso>@ in lens's terminology. For instance (assuming you don't mind functions that can error out), here's a lens which lets you act on the string representation of a value:
+
+@
+string :: (Read a, Show a) => 'Lens'' a String
+string f s = read '<$>' f (show s)
+@
+
+Using it to reverse a number:
+
+@
+>>> 123 '&' string '%~' reverse
+321
+@
+-}
+
 {- |
 'lens' creates a 'Lens' from a getter and a setter. The resulting lens isn't the most effective one (because of having to traverse the structure twice when modifying), but it shouldn't matter much.
 
@@ -734,6 +844,43 @@ non x afb s = f <$> afb (fromMaybe x s)
 
 -- Traversals --------------------------------------------------------------
 
+{- $traversals-note
+
+Traversals are like lenses but they can point at multiple values. Use ('^..') to get all values, ('^?') to get the 1st value, ('.~') to set values, ('%~') to modify them. ('.') composes traversals just as it composes lenses. ('^.') can be used with traversals as well, but don't confuse it with ('^..') – ('^..') gets all traversed values, ('^.') combines traversed values using the ('Data.Monoid.<>') operation (if the values are instances of 'Monoid'; if they aren't, it won't compile). 'traverseOf' and 'traverseOf_' apply an action to all pointed values of a traversal.
+
+Traversals don't differ from lenses when it comes to setting – you can use usual ('%~') and ('.~') to modify and set values. Getting is a bit different, because you have to decide what to do in the case of multiple values. In particular, you can use these combinators (as well as everything else in the “Folds” section):
+
+  * ('^..') gets a list of values
+  * ('^?') gets the 1st value (or 'Nothing' if there are no values)
+  * ('^?!') gets the 1st value and throws an exception if there are no values
+
+If you are sure that the traversal will traverse at least one value, you can convert it to a lens with 'singular'.
+
+'traversed' is a universal traversal for anything that belongs to the 'Traversable' typeclass. However, in many cases 'each' works as well and is shorter and nicer-looking.
+-}
+
+{- |
+Apply an action to all targets (like 'Control.Monad.mapM' or 'Data.Traversable.traverse'):
+
+>>> traverseOf both readFile ("file1", "file2")
+(<contents of file1>, <contents of file2>)
+
+>>> traverseOf _1 id (Just 1, 2)
+Just (1, 2)
+>>> traverseOf _1 id (Nothing, 2)
+Nothing
+
+You can also just apply the lens\/traversal directly (but 'traverseOf' might be more readable):
+
+>>> both readFile ("file1", "file2")
+(<contents of file1>, <contents of file2>)
+
+If you don't need the result, use 'traverseOf_'.
+-}
+traverseOf :: LensLike f s t a b -> (a -> f b) -> s -> f t
+traverseOf = id
+{-# INLINE traverseOf #-}
+
 {- |
 'singular' turns a traversal into a lens that behaves like a single-element traversal:
 
@@ -1091,6 +1238,14 @@ _Nothing _ j = pure j
 
 -- Some of the guts of lens
 
+newtype Traversed a f = Traversed { getTraversed :: f a }
+
+instance Applicative f => Monoid (Traversed a f) where
+  mempty = Traversed (pure (error "Lens.Micro.Traversed: value used"))
+  {-# INLINE mempty #-}
+  Traversed ma `mappend` Traversed mb = Traversed (ma *> mb)
+  {-# INLINE mappend #-}
+
 newtype Bazaar a b t = Bazaar (forall f. Applicative f => (a -> f b) -> f t)
 
 instance Functor (Bazaar a b) where
